#!/usr/bin/perl
use Math::Trig;
use Getopt::Long;
use File::Path;
use File::Copy;

# add 3 new mod files in LOMETS version 2 
#########################################################################check OS
$operating_system=$^O;
unless ($operating_system eq "linux"){
    printf("Your operating system $operating_system is unsupported at this time\n");
    printf("Currently only Linux is supported\n");
    exit();
}
GetOptions('libdir:s' => \$libdir, 'seqname:s' => \$seqname, 'datadir:s' => \$datadir,
	       'usrname:s' => \$usrname, 'tmptop:s' => \$tmptop, 'homoflag:s' => \$run,
	       'idcut:s' => \$id_cut, 'ntemp:s' => \$n_temp, 'ntop:s' => \$ntop,'-javadir:s' =>\$javadir);
if(!$libdir || !$seqname || !$datadir)
{
	printf "LOMETS USAGE:\n";
	printf "Mandatory arguments:\n";
	printf "./runLOMETS.pl -libdir package_direcotry -seqname sequence_name -datadir data_directory\n";
	printf "Optional arguments whose default values have been set:\n";
	printf "-usrname    the user name\n";
	printf "-tmptop	    path to the top temporary directory, default is \"tmp\"\n";
	printf "-homoflag   default is real which means using all templates, benchmark means removing homologous templates\n";
	printf "-idcut      sequence identity cutoff when using benchmark, default value is 0.30, range is (0.00,1.00)\n";
	printf "-ntemp      number of top templetes selected by each threading program, default value is 20, range is [1,50]\n";
	printf "-ntop       number of top threading alignments by each threading program, default value is 10, range is [1,20]\n";
	exit();
}	 
$run = "benchmark";      
if(!$run) 
{
	$run="real"; #'real', use all templates; 'benchmark', homology will be removed
}
if($run eq "benchmark" && !$id_cut)
{
	$id_cut=0.3;   #cut-off  of sequence idendity
}
elsif(!$id_cut)
{
	$id_cut=1.0;   #cut-off  of sequence idendity
}
if($run eq "real" && $id_cut<1.0)
{
	$id_cut=1.0;
}

if($n_temp>50 || $n_temp<1)
{
   $n_temp=20;
}

if(!$n_temp)
{
	$n_temp=20;     #number of templates per threading algorithm
}
if(!$ntop)
{
	$ntop=10;#number of threading alignments for output
}
if($ntop>$n_temp)
{
	$ntop=$n_temp;
}

if(!$usrname)
{
	$usrname=getlogin() || $ENV{'USER'};
}
if (!$tmptop)
{
	$tmptop = "tmp";
} 
if(!$javadir)
{

   $javadir="/usr/java/latest";
}
 
printf "Your setting is:\n";
printf "-libdir   =$libdir\n";
printf "-seqname  =$seqname\n";
printf "-datadir  =$datadir\n";
printf "-usrname  =$usrname\n";
printf "-tmptop   =$tmptop\n";
printf "-homoflag =$run\n";
printf "-idcut    =$id_cut\n";
printf "-ntemp    =$n_temp\n";
printf "-ntop     =$ntop\n";
printf "-javadir     =$javadir\n";

$blastdir="$libdir/blast/bin";
$db="$libdir/nr/nr";
 

#################################################################################################################################### 
%ts=(
     'GLY'=>'G',
     'ALA'=>'A',
     'VAL'=>'V',
     'LEU'=>'L',
     'ILE'=>'I',
     'SER'=>'S',
     'THR'=>'T',
     'CYS'=>'C',
     'MET'=>'M',
     'PRO'=>'P',
     'ASP'=>'D',
     'ASN'=>'N',
     'GLU'=>'E',
     'GLN'=>'Q',
     'LYS'=>'K',
     'ARG'=>'R',
     'HIS'=>'H',
     'PHE'=>'F',
     'TYR'=>'Y',
     'TRP'=>'W',

     'ASX'=>'B',
     'GLX'=>'Z',
     'UNK'=>'X',

     'G'=>'GLY',
     'A'=>'ALA',
     'V'=>'VAL',
     'L'=>'LEU',
     'I'=>'ILE',
     'S'=>'SER',
     'T'=>'THR',
     'C'=>'CYS',
     'M'=>'MET',
     'P'=>'PRO',
     'D'=>'ASP',
     'N'=>'ASN',
     'E'=>'GLU',
     'Q'=>'GLN',
     'K'=>'LYS',
     'R'=>'ARG',
     'H'=>'HIS',
     'F'=>'PHE',
     'Y'=>'TYR',
     'W'=>'TRP',

     'a'=>'CYS',
     'b'=>'CYS',
     'c'=>'CYS',
     'd'=>'CYS',
     'e'=>'CYS',
     'f'=>'CYS',
     'g'=>'CYS',
     'h'=>'CYS',
     'i'=>'CYS',
     'j'=>'CYS',
     'k'=>'CYS',
     'l'=>'CYS',
     'm'=>'CYS',
     'n'=>'CYS',
     'o'=>'CYS',
     'p'=>'CYS',
     'q'=>'CYS',
     'r'=>'CYS',
     's'=>'CYS',
     't'=>'CYS',
     'u'=>'CYS',
     'v'=>'CYS',
     'w'=>'CYS',
     'x'=>'CYS',
     'y'=>'CYS',
     'z'=>'CYS',

     'B'=>'ASX',
     'Z'=>'GLX',
     'X'=>'CYS',
    );
################# directories #############################

$time=`date`;
printf "starting time: $time";
$pwd=`pwd`;
printf "pwd: $pwd";
$unicode="LM$seqname";#useful when making tmp dir and setting unique jobname
$tmpdir="/$tmptop/$usrname/$unicode";
if(-d "$tmpdir")
{
rmtree("$tmpdir");
}
mkpath("$tmpdir");
if(!-d "$tmpdir")
{
exit();
}
#`/bin/mkdir -p $tmpdir`;
chdir "$tmpdir";
#if(!-s "$tmpdir")
#{
#	printf "cannot make temporary directory\n";
#	exit();
#}
#`/bin/rm -fr $tmpdir/*`;#remove all thing in working directory

LMposition1:;
printf "1. make seq.txt\n";
############convert 'seq.fasta' to 'seq.txt' with standard format ####
if(-s "$datadir/seq.txt" && -s "$datadir/rmsinp")
{
	goto LMposition2;
}
if(!-s "$datadir/seq.fasta")
{
	printf "error: without $datadir/seq.fasta\n";
	goto LMpositionend;
}

open(seqtxt,"$datadir/seq.fasta");
$sequence="";
while($line=<seqtxt>)
{
	goto pos1_1 if($line=~/^>/);
	if($line=~/(\S+)/)
	{
		$sequence .=$1;
	}
      	pos1_1:;
    }
close(seqtxt);
$Lch=length $sequence;
if($Lch==0){
	printf "error: the length is zero\n";
	goto LMpositionend;
}
open(fasta,">$datadir/seq.txt");
printf fasta "> $seqname\n";
for($i=1;$i<=$Lch;$i++){
	$seq1=substr($sequence,$i-1,1);
	$seq2=$ts{$seq1};
    	$seq1=$ts{$seq2};
	printf fasta "$seq1";
	if(int($i/60)*60==$i){
		printf fasta "\n";
	}
}
printf fasta "\n";
close(fasta);
open(rmsinp,">$datadir/rmsinp");
printf rmsinp "1  $Lch\n";
printf rmsinp "$Lch\n";
printf rmsinp "protein\n";
close(rmsinp);

LMposition2:;
copy("$datadir/seq.txt","$tmpdir/psitmp.fasta");
########### run psi-blast #######################
printf "2.1 run Psi-blast\n";
if(-s "$datadir/psitmp.chk" && -s "$datadir/blast.out" && -s "$datadir/pssm.txt"){
	goto pos2_1;
}else{
	`$blastdir/blastpgp  -b 1000 -j 3 -h 0.001 -d $db -i psitmp.fasta -C psitmp.chk -Q pssm.txt > blast.out`;
	copy("psitmp.chk","$datadir/psitmp.chk");
	copy("blast.out","$datadir/blast.out");
	copy("pssm.txt","$datadir/pssm.txt");
}
pos2_1:;
########### make mtx ####################
if(-s "$datadir/mtx")
{
	goto pos2_2;     
}
if(!-s "$tmpdir/psitmp.chk")
{
	copy("$datadir/psitmp.chk","$tmpdir/psitmp.chk");
}
#`echo psitmp.fasta > psitmp.sn`;
#`echo psitmp.chk > psitmp.pn`;
open(pn, ">psitmp.pn");
printf pn "psitmp.chk\n";
close(pn);
open(pn, ">psitmp.sn");
printf pn "psitmp.fasta\n";
close(pn);
`$blastdir/makemat -P psitmp`;;
copy("psitmp.mtx","$datadir/mtx");
########### run PSSpred ####################
pos2_2:;
printf "2.2 run PSSpred\n";
if(-s "$datadir/seq.dat" && -s "$datadir/seq.dat.ss"){
	goto LMposition3;
}
if(!-s "$tmpdir/psitmp.chk"){
	copy("$datadir/psitmp.chk","$tmpdir/psitmp.chk");
}
if(!-s "$tmpdir/mtx"){
	copy("$datadir/mtx","$tmpdir/mtx");
}
if(!-s "$tmpdir/blast.out"){
	copy("$datadir/blast.out","$tmpdir/blast.out");
}
if(!-s "$tmpdir/pssm.txt"){
	copy("$datadir/pssm.txt","$tmpdir/pssm.txt");
}

if(-e "$datadir/seq.fasta"){
	`cp $datadir/seq.fasta .`;
}elsif(-e "$datadir/seq.txt"){
	`cp $datadir/seq.txt .`;
}

if(-s "seq.fasta"){
        `perl $libdir/PSSpred/PSSpred.pl seq.fasta $libdir`;
}elsif(-s "seq.txt"){
        `perl $libdir/PSSpred/PSSpred.pl seq.txt   $libdir`;
}else{
        printf "error: without $datadir/seq.fasta && $datadir/seq.txt\n";
        goto ITpositionend;
}
`cp mtx        $datadir/`;
`cp blast.out  $datadir/`;
`cp pssm.txt   $datadir/`;
`cp psitmp.chk $datadir/`;
`cp seq.dat    $datadir/`;
`cp seq.dat.ss $datadir/seq.ss`;

copy("seq.dat","$datadir/seq.dat");
copy("seq.dat.ss","$datadir/seq.dat.ss");

LMposition3:;
printf "3.1 do threading\n";
@TT=qw(
PPA
dPPA
dPPA2
sPPA
MUSTER
wPPA
wdPPA
wMUSTER
); #threading programs
$totinit=0;
foreach $T(@TT)
{
	if(-s "$datadir/init.$T")
	{
		$totinit++;
	}
}	
if($totinit==8)
{
	goto pos3_1;
}	
foreach $T(@TT)
{
	print "$datadir/init.$T\n";
	if(!-s "$datadir/init.$T")
	{
		$tag="$T\_$seqname"; # unique name
	    	$jobmod="$T"."mod";
		$jobname="$datadir/$tag";
		#------- jobname ------>
	#	$mod=`cat $libdir/I-TASSERmod/$jobmod`;
		open(modtxt,"$libdir/I-TASSERmod/$jobmod");
                $mod="";
                while($line=<modtxt>)
                {
                      $mod.=$line;
                }
                close(modtxt);
		$mod=~s/\!S\!/$seqname/mg;
		$mod=~s/\!O\!//mg;
		$mod=~s/\!ID_CUT\!/$id_cut/mg;
		$mod=~s/\!N_TEMP\!/$n_temp/mg;
		$mod=~s/\!DATA_DIR\!/$datadir/mg;
		$mod=~s/\!DATADIR\!/$datadir/mg;
		$mod=~s/\!LIB_DIR\!/$libdir/mg;
		$mod=~s/\!TAG\!/$tag/mg;
		$mod=~s/\!USER\!/$usrname/mg;
		$mod=~s/\!TMPTOP\!/$tmptop/mg;
		$mod=~s/\!BLASTDIR\!/$blastdir/mg;
		$mod=~s/\!NRLIB\!/$db/mg;
		$mod=~s/\!RUN\!/$run/mg;
		$mod=~s/\!JAVA_PATH\!/$javadir/mg;

		open(job,">$jobname");
		print job "$mod\n";
		close(job);
	#	`chmod a+x $jobname`;
		chmod 0744, "$jobname";
		printf "start threading $T\n";
		$rst=`$jobname`;
		printf "$rst";
	}
}
$totinit=0;
foreach $T(@TT)
{
	if(-s "$datadir/init.$T")
	{
		$totinit++;
	}
}
if($totinit<4)
{
	printf "only $totinit threading programs have output\n";
	goto LMpositionend;
}
pos3_1:;
printf "3.2 make init.dat\n";
if(not -s "$datadir/init.dat" )
{
#generate init.dat 
`$libdir/I-TASSERmod/mkinit4ThreaDom.pl $libdir $datadir`;
}

if(!-s "$datadir/init.dat" )
{
	printf "missing some dat file after threading\n";
	goto LMpositionend;
}

############## remove the directory ################
LMpositionend:;
printf "Jobs finished!\n";
$time=`date`;
printf "ending time: $time";
`sync`;
`sync`;
sleep(2);
#`/bin/rm -fr $tmpdir`;
rmtree("$tmpdir");
exit();
 

